@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 2002-2006, 2009, 2010, 2013, 2014 by                          */
+/*  Copyright 2002-2016 by                                                  */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*  ftbench: bench some common FreeType call paths                          */
@@ -31,7 +31,9 @@
 #include FT_TRUETYPE_DRIVER_H
 
 #ifdef UNIX
-#include <sys/time.h>
+#include <unistd.h>
+#else
+#include "mlgetopt.h"
 #endif
 
 #include "common.h"
@@ -159,22 +161,27 @@
 
 
   /*
-   * timer
+   * timer in milliseconds
    */
 
   static double
   get_time( void )
   {
-#ifdef UNIX
-    struct timeval  tv;
+#if defined _POSIX_TIMERS && _POSIX_TIMERS > 0
+    struct timespec  tv;
 
 
-    gettimeofday( &tv, NULL );
-    return (double)tv.tv_sec + (double)tv.tv_usec / 1E6;
+#ifdef _POSIX_CPUTIME
+    clock_gettime( CLOCK_PROCESS_CPUTIME_ID, &tv );
 #else
-    /* clock() has an awful precision (~10ms) under Linux 2.4 + glibc 2.2 */
-    return (double)clock() / (double)CLOCKS_PER_SEC;
-#endif
+    clock_gettime( CLOCK_REALTIME, &tv );
+#endif /* _POSIX_CPUTIME */
+
+    return 1E6 * (double)tv.tv_sec + 1E-3 * (double)tv.tv_nsec;
+#else
+    /* clock() accuracy has improved since glibc 2.18 */
+    return 1E6 * (double)clock() / (double)CLOCKS_PER_SEC;
+#endif /* _POSIX_TIMERS */
   }
 
 #define TIMER_START( timer )  ( timer )->t0 = get_time()
@@ -213,11 +220,10 @@
     printf( "  %-25s ", test->title );
     fflush( stdout );
 
-    n = done = 0;
     TIMER_RESET( &timer );
     TIMER_RESET( &elapsed );
 
-    for ( n = 0; !max_iter || n < max_iter; n++ )
+    for ( n = 0, done = 0; !max_iter || n < max_iter; n++ )
     {
       TIMER_START( &elapsed );
 
@@ -225,12 +231,12 @@
 
       TIMER_STOP( &elapsed );
 
-      if ( TIMER_GET( &elapsed ) > max_time )
+      if ( TIMER_GET( &elapsed ) > 1E6 * max_time )
         break;
     }
 
     if ( done )
-      printf( "%5.3f us/op\n", TIMER_GET( &timer ) * 1E6 / (double)done );
+      printf( "%5.3f us/op\n", TIMER_GET( &timer ) / (double)done );
     else
       printf( "no error-free calls\n" );
   }
